"""{{ mixedName }}.py - Module containing the {{ mixedName }} class"""
from .{{gensubdir}}.{{ genprefix }}{{ mixedName }} import {{ genprefix }}{{ mixedName }}

class {{ mixedName }}({{ genprefix }}{{ mixedName }}):
    pass
