# Ignore virtualenv files
lib/**
man/**
include/**
bin/**
.Python

# Ignore coverage files
coverage/**

# Ignore python cache files
src/api/__pycache__/**
src/dm/__pycache__/**
src/dm/smoacks/__pycache__/**
src/util/__pycache__/**
tests/__pycache__/**

# Ignore templates (should likely be configurable)
templates/**
