# Attempt to better manage Flask logging
version: 1
formatters:
  simple:
    format: '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
handlers:
  console:
    class: logging.StreamHandler
    level: DEBUG
    formatter: simple
    stream: ext://sys.stdout
loggers:
  appLogger:
    level: INFO
    handlers: [console]
    propagate: no
  requestLogger:
    level: ERROR
    handlers: [console]
    propagate: no
  connexion.api:
    level: DEBUG
    handlers: [console]
    propagate: no
root:
  level: DEBUG
  handlers: [console]
